/*
Interactive Wall for Artist in Residence Show
Author: Jason Webb
Date: September 2011
URL: http://cs.unk.edu/~webb/2011/09/interactive-wall-for-artist-in-residence-show/

Description: Sketch to control an array of 24VDC pull-type solenoids using momentary 
switches connected to a Teensy. When a switch is held down, it's corresponding buffer
begins to fill. When buffer is full, appropriate solenoid is triggered and buffer is
reset.

** Note: This piece was originally going to use an EL Escudo board from SparkFun.com
to control 6 EL wires attached to the wall. But due to complications in installation,
this never happened. I left the functionality in anyway, since it didn't hurt anything,
and I might yet figure out what went wrong. If you don't want to use EL wire, remove the 
first line of the program (#include line) and clean up any errors that result.
*/

#include <EL_Escudo.h>

/** PROGRAM CONSTANTS *****************************************************************/
#define  NUM_BUTTONS    10   // Number of momentary buttons
#define  NUM_SWITCHES   5    // Number of toggle switches
#define  NUM_SOLENOIDS  10   // Number of solenoids
#define  NUM_EL_WIRES   5    // Number of EL wires
#define  SOLENOID_TIME  100  // Time to activate solenoid (ms)
#define  BUFFER_SIZE    500  // Size of buffer (number of loops to wait between solenoid strikes)

boolean DEBUG = true;  // Print out info via serial

/** PROGRAM VARIABLES *****************************************************************/
// Momentary on buttons
int BUTTON_PINS[10] = {PIN_B7, PIN_D0, PIN_D1, PIN_D2, PIN_D3, PIN_D4, PIN_D5, PIN_D6, PIN_D7, PIN_E0};
int BUTTON_VALUES[10];
int BUTTON_BUFFER[10];

// Toggle switches
int SWITCH_PINS[10] = {PIN_E1, PIN_C0, PIN_C1, PIN_C2, PIN_C3};
int SWITCH_VALUES[10];

// Solenoid thwwackers
int SOLENOID_PINS[10] = {PIN_B6, PIN_B5, PIN_B4, PIN_B3, PIN_B2, PIN_B1, PIN_B0, PIN_E7, PIN_E6, PIN_F0};

/** MAIN PROGRAM SETUP ******************************************************************/
void setup() {
  // Set up all the momentary buttons pins
  for(int i=0; i<NUM_BUTTONS; i++)
    pinMode(BUTTON_PINS[i], INPUT);
  
  // Set up all the toggle switches pins
  for(int i=0; i<NUM_SWITCHES; i++)
    pinMode(SWITCH_PINS[i], INPUT);
    
  // Set up all the solenoid pins
  for(int i=0; i<NUM_SOLENOIDS; i++)
    pinMode(SOLENOID_PINS[i], OUTPUT);
    
  // Set up and initialize all EL wires to off
  EL.all_off();
  
  // Initialize button buffer
  for(int i=0; i<NUM_BUTTONS; i++)
    BUTTON_BUFFER[i] = 0;
  
  // Begin serial communication
  if(DEBUG)  Serial.begin(9600);
}

/** MAIN PROGRAM LOOP *********************************************************************/
void loop() {
  
  // Read all the momentary button states
  for(int i=0; i<NUM_BUTTONS; i++)
    BUTTON_VALUES[i] = !digitalRead(BUTTON_PINS[i]);
    
  // Read all the toggle switch states
  for(int i=0; i<NUM_SWITCHES; i++)
    SWITCH_VALUES[i] = digitalRead(SWITCH_PINS[i]);

      // Print all of the switch states  
      if(DEBUG) {
        for(int i=0; i<NUM_BUTTONS; i++) {
          Serial.
          print( BUTTON_VALUES[i] );
          Serial.print("\t");
        }
      
        Serial.println();
      }

  // Energize solenoids when buttons are pressed
  for(int i=0; i<NUM_BUTTONS; i++) {
    // If a button is detect as 'on' ...
    if( BUTTON_VALUES[i] == 1 ) {
      
      // Increment button's buffer until it overflows ...
      if( BUTTON_BUFFER[i] >= 0 && BUTTON_BUFFER[i] < BUFFER_SIZE )
        BUTTON_BUFFER[i]++;
      else
        BUTTON_BUFFER[i] = 0;
      
      // Strike the correct solenoid when the buffer overflows
      if( BUTTON_BUFFER[i] == 0 )
        activateSolenoid(i);
    }
  }
  
  // Manage the EL wires based on the toggle switch inputs
  for(int i=0; i<NUM_SWITCHES; i++) {
    if( SWITCH_VALUES[i] == 1 ) {
      switch(i) {
        case 0:
          EL.on(A);
          break;
        case 1:
          EL.on(B);
          break;
        case 2:
          EL.on(C);
          EL.on(D);
          break;
        case 3:
          EL.on(E);
          break;
        case 4:
          EL.on(F);
          break;
      }
    } else {
      switch(i) {
        case 0:
          EL.off(A);
          break;
        case 1:
          EL.off(B);
          break;
        case 2:
          EL.off(C);
          EL.off(D);
          break;
        case 3:
          EL.off(E);
          break;
        case 4:
          EL.off(F);
          break;
      }      
    }
      
  }
  
}

//------------------------------------------------
// Activates a single solenoid
//------------------------------------------------
void activateSolenoid( int index ) {
  digitalWrite( SOLENOID_PINS[index], HIGH );
  delay(SOLENOID_TIME);
  digitalWrite( SOLENOID_PINS[index], LOW );
}
